@@ -105,7 +105,7 @@ final public class GraphicDisplay extends javax.swing.JPanel {
             }
             if (parent.drawGrid) {
                 rayTraceComputer.drawGrid(bg);
-                rayTraceComputer.drawBaseline(bg);
+                rayTraceComputer.drawBaselines(bg);
             }
             if (parent.antiAlias) {
                 RenderingHints rh = new RenderingHints(
@@ -166,6 +166,7 @@ final public class GraphicDisplay extends javax.swing.JPanel {
         if (!inside) {
             updateStatusBar(0, 0, true);
         }
+        parent.mouse_present_test(inside);
     }
 
     void updateDisplay() {
