@@ -30,7 +30,6 @@
 package opticalraytracer;
 
 import javax.swing.*;
-import java.net.*;
 import java.util.*;
 import java.awt.*;
 import java.awt.image.*;
@@ -40,25 +39,20 @@ import java.awt.datatransfer.DataFlavor;
 import java.awt.datatransfer.ClipboardOwner;
 import java.awt.datatransfer.Transferable;
 import java.text.*;
+import javax.swing.text.html.parser.*;
+import netscape.javascript.*;
 
 /**
  *
  * @author lutusp
  */
-final public class OpticalRayTracer extends javax.swing.JFrame implements ClipboardOwner {
+final public class OpticalRayTracer extends javax.swing.JPanel implements ClipboardOwner {
 
-    final String appVersion = "2.8";
+    final String appVersion = "3.2";
     final String appName;
     final String programName;
-    boolean applet = false;
-    //String appPath;
-    //String fileSep;
-    //String lineSep;
-    //String extPath;
-    //String userDir;
-    //String userPath;
-    //String dataPath;
-    JFrame sv_programFrame;
+    JApplet applet = null;
+    JFrame sv_programFrame = null;
     MyHelpPane helpPane;
     NumberFormat numberFormat;
     int sv_helpScrollPos = 0;
@@ -110,7 +104,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
     boolean antiAlias;
     boolean inverted;
     boolean drawGrid;
-
+    boolean rotXZero = false;
     // END default initializations
     double sv_lensMinThickness; // % of lens radius
     double userThickness;
@@ -123,27 +117,35 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
     int popupMouseX, popupMouseY;
 
     /** Creates new form OpticalRayTracer */
-    public OpticalRayTracer() {
+    public OpticalRayTracer(JFrame frame, JApplet applet, String[] args) {
+        this.applet = applet;
+        sv_programFrame = (JFrame) frame;
+        appName = getClass().getSimpleName();
+        // this is essential to avoid a problem with
+        // HTMLEditorKit and applets
+        ParserDelegator workaround = new ParserDelegator();
         initComponents();
         // just for testing locale issues
         //Locale.setDefault(Locale.GERMANY);
         numberFormat = NumberFormat.getNumberInstance();
         sv_lensList = new Vector<Lens>();
         resetUndoRedo();
-        sv_programFrame = this; // used to set screen geometry
+
+        //sv_programFrame = this; // used to set screen geometry
         // Just for the first run
         int w = 800;
         int h = 650;
         Dimension ss = Toolkit.getDefaultToolkit().getScreenSize();
-        setBounds(new Rectangle((ss.width - w) / 2, (ss.height - h) / 2, w, h));
-        appName = getClass().getSimpleName();
-        URL url = getClass().getResource(appName + ".class");
+        //URL url = getClass().getResource(appName + ".class");
         //appPath = url.getPath().replaceFirst("(.*?)!.*", "$1");
         //appPath = appPath.replaceFirst("file:", "");
         //appPath = new File(appPath).getPath();
         programName = appName + " " + appVersion;
-        setTitle(programName);
-        setIconImage(new ImageIcon(getClass().getResource("icons/" + appName + ".png")).getImage());
+        if (sv_programFrame != null) {
+            sv_programFrame.setBounds(new Rectangle((ss.width - w) / 2, (ss.height - h) / 2, w, h));
+            //sv_programFrame.setTitle(programName);
+            //sv_programFrame.setIconImage(new ImageIcon(getClass().getResource("icons/" + appName + ".png")).getImage());
+        }
         //userDir = System.getProperty("user.home");
         //lineSep = System.getProperty("line.separator");
         //fileSep = System.getProperty("file.separator");
@@ -162,8 +164,12 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
 
         //setupColorButtons();
 
-        initManager = new InitManager(this);
-        initManager.readConfig();
+        initManager = new InitManager(this, applet != null);
+        if (applet == null) {
+            initManager.readConfig();
+        }
+        this.quitButton.setEnabled(applet == null);
+        this.copyImageButton.setEnabled(applet == null);
         if (sv_selectedLensIndex != -1 && sv_selectedLensIndex < sv_lensList.size()) {
             setSelectedLens(sv_lensList.get(sv_selectedLensIndex));
         }
@@ -179,10 +185,37 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
             }
         });
         makeDefaultLenses(false);
+        // this forces tooltips to be shown in applet mode
+        mouse_present_test(true);
+    }
+
+    public String getConfig() {
+        if (initManager != null) {
+            sv_helpScrollPos = helpPane.getScrollPos();
+            return initManager.getConfig();
+        } else {
+            return null;
+        }
+    }
+
+    public void setConfig(String data) {
+        initManager.setConfig(data);
+        readControls();
+        //enabled = true;
+        //setModeAndLabels();
+        //mainPanel3d.updateSettings();
+        processTabClick();
+        SwingUtilities.invokeLater(
+                new Runnable() {
+
+                    public void run() {
+                        helpPane.setScrollPos(sv_helpScrollPos);
+                    }
+                });
     }
 
     // debugging printers
-    void p(String s) {
+    public <T> void p(T s) {
         System.out.println(s);
     }
 
@@ -267,16 +300,16 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         new UserActionManager(textFieldDoubleSensitivity, -1e10, 1e10, yPosTextField, this);
 
         new UserActionManager(textFieldIntSensitivity, -1e10, 1e10, sv_dotRadiusTextField, this);
-        new UserActionManager(textFieldDoubleSensitivity, -1e10, 1e10, sv_snapToBaseTextField, this);
+        new UserActionManager(textFieldDoubleSensitivity, 0, 1e10, sv_snapToBaseTextField, this);
         new UserActionManager(textFieldDoubleSensitivity, -1e10, 1e10, sv_beamWidthTextField, this);
-        new UserActionManager(textFieldIntSensitivity, -1e10, 1e10, sv_beamCountTextField, this);
-        new UserActionManager(textFieldIntSensitivity, -1e10, 1e10, sv_intersectionsTextField, this);
+        new UserActionManager(textFieldIntSensitivity, 2, 1e10, sv_beamCountTextField, this);
+        new UserActionManager(textFieldIntSensitivity, 0, 1e10, sv_intersectionsTextField, this);
         new UserActionManager(textFieldDoubleSensitivity, -1e10, 1e10, sv_yStartTextField, this);
         new UserActionManager(textFieldDoubleSensitivity, -1e10, 1e10, sv_yEndTextField, this);
         new UserActionManager(textFieldDoubleSensitivity, -1e10, 1e10, sv_xSourcePlaneTextField, this);
         new UserActionManager(textFieldDoubleSensitivity, -1e10, 1e10, sv_xTargetPlaneTextField, this);
-        new UserActionManager(textFieldDoubleSensitivity, -1e10, 1e10, sv_offsetAngleTextField, this);
-        new UserActionManager(textFieldIntSensitivity, -1e10, 1e10, sv_dispersionBeamsTextField, this);
+        new UserActionManager(textFieldIntSensitivity, -1e10, 1e10, sv_offsetAngleTextField, this);
+        new UserActionManager(textFieldIntSensitivity, 0, 1e10, sv_dispersionBeamsTextField, this);
     }
 
     public void updateGraphicDisplay() {
@@ -351,7 +384,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
     }
 
     void setupColorButtons() {
-        sv_yBaselineColor = new ColorButton(this, baselineColorButton, 0x004000, "Y Baseline");
+        sv_yBaselineColor = new ColorButton(this, baselineColorButton, 0x004000, "X/Y Baseline");
         sv_gridColor = new ColorButton(this, gridColorButton, 0xc0c0c0, "Grid");
         sv_lensOutlineColor = new ColorButton(this, lensOutlineColorButton, 0x80a0ff, "Lens outline");
         sv_intersectionColor = new ColorButton(this, intersectionColorButton, 0xaa00aa, "Beam intersection");
@@ -425,7 +458,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
             }
         } catch (Exception ex) {
             System.out.println(getClass().getName() + ".getDouble: Error: " + ex + ", source: " + s);
-        //ex.printStackTrace();
+            //ex.printStackTrace();
         }
         return v;
     }
@@ -472,6 +505,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         drawGrid = sv_gridCheckBox.isSelected();
         antiAlias = sv_antiAliasCheckBox.isSelected();
         inverted = sv_invertedCheckBox.isSelected();
+        rotXZero = sv_zeroRotCheckBox.isSelected();
         updateGraphicDisplay();
     }
 
@@ -491,6 +525,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         sv_gridCheckBox.setSelected(drawGrid);
         sv_antiAliasCheckBox.setSelected(antiAlias);
         sv_invertedCheckBox.setSelected(inverted);
+        sv_zeroRotCheckBox.setSelected(rotXZero);
 
     }
 
@@ -516,10 +551,12 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
     }
 
     void clipboardCopyImage() {
-        this.gd1.rayTraceProcessCore(sv_clipboardGraphicXSize, this.sv_clipboardGraphicYSize);
-        ImageTransferable imt = new ImageTransferable(image);
-        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
-        clipboard.setContents(imt, null);
+        if (applet == null) {
+            this.gd1.rayTraceProcessCore(sv_clipboardGraphicXSize, this.sv_clipboardGraphicYSize);
+            ImageTransferable imt = new ImageTransferable(image);
+            Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
+            clipboard.setContents(imt, null);
+        }
     }
 
     void clipboardCopyFullConfig() {
@@ -538,20 +575,45 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
     }
 
     void setClipboardContents(String s) {
-        StringSelection stringSelection = new StringSelection(s);
-        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
-        clipboard.setContents(stringSelection, this);
+        if (applet != null) {
+            try {
+                //this.applet_clipboard_textpane.setText(s);
+                JSObject window = JSObject.getWindow(applet);
+                JSObject doc = (JSObject) window.getMember("document");
+                JSObject form = (JSObject) doc.getMember("clipform");
+                JSObject field = (JSObject) form.getMember("cliparea");
+                field.setMember("value", s);
+            } catch (Exception e) {
+                System.out.println("setClipboardContents: " + e);
+            }
+        } else {
+            StringSelection stringSelection = new StringSelection(s);
+            Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
+            clipboard.setContents(stringSelection, this);
+        }
     }
 
     String getClipboardContents() {
         String s = null;
-        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
-        Transferable content = clipboard.getContents(this);
-        if (content != null) {
+        if (applet != null) {
             try {
-                s = (String) content.getTransferData(DataFlavor.stringFlavor);
+                JSObject window = JSObject.getWindow(applet);
+                JSObject doc = (JSObject) window.getMember("document");
+                JSObject form = (JSObject) doc.getMember("clipform");
+                JSObject field = (JSObject) form.getMember("cliparea");
+                s = (String) field.getMember("value");
             } catch (Exception e) {
-                //System.out.println("getClipboardContents: " + e);
+                System.out.println("getClipboardContents1: " + e);
+            }
+        } else {
+            Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
+            Transferable content = clipboard.getContents(this);
+            if (content != null) {
+                try {
+                    s = (String) content.getTransferData(DataFlavor.stringFlavor);
+                } catch (Exception e) {
+                    System.out.println("getClipboardContents2: " + e);
+                }
             }
         }
         return s;
@@ -623,10 +685,21 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         }
     }
 
+    // this allows tooltips to be displayed
+    // in the applet mode after a mouse exit/enter
+    public void mouse_present_test(boolean present) {
+        if (present) {
+            this.requestFocus();
+        }
+
+    }
+
     public void close() {
-        this.sv_helpScrollPos = helpPane.getScrollPos();
-        sv_selectedLensIndex = sv_lensList.indexOf(selectedLens);
-        initManager.writeConfig();
+        if (applet == null) {
+            this.sv_helpScrollPos = helpPane.getScrollPos();
+            sv_selectedLensIndex = sv_lensList.indexOf(selectedLens);
+            initManager.writeConfig();
+        }
         setVisible(false);
         System.exit(0);
     }
@@ -723,16 +796,19 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         jLabel12 = new javax.swing.JLabel();
         sv_dispersionBeamsTextField = new javax.swing.JTextField();
         sv_divergingSourceCheckBox = new javax.swing.JCheckBox();
+        sv_zeroRotCheckBox = new javax.swing.JCheckBox();
         helpTab = new javax.swing.JPanel();
         statusLabel = new javax.swing.JLabel();
 
-        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
-        addWindowListener(new java.awt.event.WindowAdapter() {
-            public void windowClosing(java.awt.event.WindowEvent evt) {
-                formWindowClosing(evt);
+        addMouseListener(new java.awt.event.MouseAdapter() {
+            public void mouseEntered(java.awt.event.MouseEvent evt) {
+                formMouseEntered(evt);
+            }
+            public void mouseExited(java.awt.event.MouseEvent evt) {
+                formMouseExited(evt);
             }
         });
-        getContentPane().setLayout(new java.awt.GridBagLayout());
+        setLayout(new java.awt.GridBagLayout());
 
         sv_mainTabbedPane.setTabPlacement(javax.swing.JTabbedPane.BOTTOM);
         sv_mainTabbedPane.addMouseListener(new java.awt.event.MouseAdapter() {
@@ -1030,6 +1106,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         leftRadiusTextField.setFont(new java.awt.Font("Monospaced", 0, 14));
         leftRadiusTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
         leftRadiusTextField.setText("0.0");
+        leftRadiusTextField.setToolTipText("<html>The radius of an imaginary sphere<br/>\nof which this lens is a part</html>");
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 1;
         gridBagConstraints.gridy = 1;
@@ -1042,6 +1119,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         rightRadiusTextField.setFont(new java.awt.Font("Monospaced", 0, 14));
         rightRadiusTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
         rightRadiusTextField.setText("0.0");
+        rightRadiusTextField.setToolTipText("<html>The radius of an imaginary sphere<br/>\nof which this lens is a part</html>");
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 1;
         gridBagConstraints.gridy = 2;
@@ -1054,6 +1132,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         leftCFTextField.setFont(new java.awt.Font("Monospaced", 0, 14));
         leftCFTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
         leftCFTextField.setText("0.0");
+        leftCFTextField.setToolTipText("<html>Hyperboloid curvature factor<br/>\n0 &lt;= f &lt;= 0.25\n</html>\n");
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 2;
         gridBagConstraints.gridy = 1;
@@ -1066,6 +1145,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         rightCFTextField.setFont(new java.awt.Font("Monospaced", 0, 14));
         rightCFTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
         rightCFTextField.setText("0.0");
+        rightCFTextField.setToolTipText("<html>Hyperboloid curvature factor<br/>\n0 &lt;= f &lt;= 0.25\n</html>");
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 2;
         gridBagConstraints.gridy = 2;
@@ -1076,6 +1156,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         controlPanel.add(rightCFTextField, gridBagConstraints);
 
         leftHypCheckBox.setBackground(new java.awt.Color(198, 238, 254));
+        leftHypCheckBox.setToolTipText("Select hyperbolic curvature");
         leftHypCheckBox.setAlignmentX(0.5F);
         leftHypCheckBox.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
         leftHypCheckBox.addMouseListener(new java.awt.event.MouseAdapter() {
@@ -1091,6 +1172,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         controlPanel.add(leftHypCheckBox, gridBagConstraints);
 
         rightHypCheckBox.setBackground(new java.awt.Color(198, 238, 254));
+        rightHypCheckBox.setToolTipText("Select hyperbolic curvature");
         rightHypCheckBox.setAlignmentX(0.5F);
         rightHypCheckBox.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
         rightHypCheckBox.addMouseListener(new java.awt.event.MouseAdapter() {
@@ -1105,7 +1187,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         gridBagConstraints.weightx = 1.0;
         controlPanel.add(rightHypCheckBox, gridBagConstraints);
 
-        hypLabel.setFont(new java.awt.Font("DejaVu Sans", 1, 14)); // NOI18N
+        hypLabel.setFont(new java.awt.Font("DejaVu Sans", 1, 14));
         hypLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
         hypLabel.setText("Hyperboloid");
         hypLabel.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
@@ -1160,7 +1242,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         dispersionTextField.setFont(new java.awt.Font("Monospaced", 0, 14));
         dispersionTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
         dispersionTextField.setText("0.0");
-        dispersionTextField.setToolTipText("A wavelength-sensitive property of some media");
+        dispersionTextField.setToolTipText("<html>A wavelength-sensitive<br/>\nproperty of some media</html>\n");
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 7;
         gridBagConstraints.gridy = 0;
@@ -1204,6 +1286,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
 
         userThicknessTextField.setHorizontalAlignment(javax.swing.JTextField.RIGHT);
         userThicknessTextField.setText("0.0");
+        userThicknessTextField.setToolTipText("Lens thickness apart from curvature");
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 3;
         gridBagConstraints.gridy = 0;
@@ -1220,7 +1303,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         gridBagConstraints.weightx = 1.0;
         controlPanel.add(controlSubPanel, gridBagConstraints);
 
-        posLabel.setFont(new java.awt.Font("DejaVu Sans", 1, 14)); // NOI18N
+        posLabel.setFont(new java.awt.Font("DejaVu Sans", 1, 14));
         posLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
         posLabel.setText("Position");
         gridBagConstraints = new java.awt.GridBagConstraints();
@@ -1282,6 +1365,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
 
         symmCheckBox.setBackground(new java.awt.Color(198, 238, 254));
         symmCheckBox.setText("Symmetrical");
+        symmCheckBox.setToolTipText("Make both lens sides the same");
         symmCheckBox.addMouseListener(new java.awt.event.MouseAdapter() {
             public void mouseClicked(java.awt.event.MouseEvent evt) {
                 symmCheckBoxMouseClicked(evt);
@@ -1647,14 +1731,26 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
             }
         });
         gridBagConstraints = new java.awt.GridBagConstraints();
-        gridBagConstraints.gridx = 4;
-        gridBagConstraints.gridy = 3;
-        gridBagConstraints.gridwidth = 2;
+        gridBagConstraints.gridx = 6;
+        gridBagConstraints.gridy = 1;
         gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
         gridBagConstraints.weightx = 1.0;
         gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
         numericPanel.add(sv_divergingSourceCheckBox, gridBagConstraints);
 
+        sv_zeroRotCheckBox.setBackground(new java.awt.Color(209, 246, 234));
+        sv_zeroRotCheckBox.setText("Rotate from X Origin");
+        sv_zeroRotCheckBox.addMouseListener(new java.awt.event.MouseAdapter() {
+            public void mouseClicked(java.awt.event.MouseEvent evt) {
+                sv_zeroRotCheckBoxMouseClicked(evt);
+            }
+        });
+        gridBagConstraints = new java.awt.GridBagConstraints();
+        gridBagConstraints.gridx = 6;
+        gridBagConstraints.gridy = 0;
+        gridBagConstraints.insets = new java.awt.Insets(2, 2, 2, 2);
+        numericPanel.add(sv_zeroRotCheckBox, gridBagConstraints);
+
         gridBagConstraints = new java.awt.GridBagConstraints();
         gridBagConstraints.gridx = 0;
         gridBagConstraints.gridy = 2;
@@ -1673,7 +1769,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         gridBagConstraints.weightx = 1.0;
         gridBagConstraints.weighty = 1.0;
         gridBagConstraints.insets = new java.awt.Insets(1, 1, 1, 1);
-        getContentPane().add(sv_mainTabbedPane, gridBagConstraints);
+        add(sv_mainTabbedPane, gridBagConstraints);
 
         statusLabel.setFont(new java.awt.Font("Monospaced", 1, 14));
         statusLabel.setText("Cursor Position");
@@ -1683,9 +1779,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
         gridBagConstraints.weightx = 1.0;
         gridBagConstraints.insets = new java.awt.Insets(2, 4, 2, 0);
-        getContentPane().add(statusLabel, gridBagConstraints);
-
-        pack();
+        add(statusLabel, gridBagConstraints);
     }// </editor-fold>//GEN-END:initComponents
 
     private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
@@ -1724,8 +1818,9 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
 }//GEN-LAST:event_sv_gridCheckBoxMouseClicked
 
     private void unselectButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_unselectButtonMouseClicked
-        // TODO add your handling code here:
-        unSelectLens();
+        if (((JButton) evt.getSource()).isEnabled()) {
+            unSelectLens();
+        }
     }//GEN-LAST:event_unselectButtonMouseClicked
 
     private void undoButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_undoButtonMouseClicked
@@ -1759,8 +1854,9 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
     }//GEN-LAST:event_sv_divergingSourceCheckBoxMouseClicked
 
     private void copyImageButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_copyImageButtonMouseClicked
-        // TODO add your handling code here:
-        clipboardCopyImage();
+        if (((JButton) evt.getSource()).isEnabled()) {
+            clipboardCopyImage();
+        }
     }//GEN-LAST:event_copyImageButtonMouseClicked
 
     private void copyFullConfigButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_copyFullConfigButtonMouseClicked
@@ -1774,8 +1870,9 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
     }//GEN-LAST:event_pasteFullConfigButtonMouseClicked
 
     private void quitButtonMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_quitButtonMouseClicked
-        // TODO add your handling code here:
-        close();
+        if (((JButton) evt.getSource()).isEnabled()) {
+            close();
+        }
     }//GEN-LAST:event_quitButtonMouseClicked
 
     private void sv_mainTabbedPaneMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_sv_mainTabbedPaneMouseClicked
@@ -1783,26 +1880,17 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
         processTabClick();
     }//GEN-LAST:event_sv_mainTabbedPaneMouseClicked
 
-    /**
-     * @param args the command line arguments
-     */
-    public static void main(String args[]) {
+    private void formMouseEntered(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseEntered
+        mouse_present_test(true);
+    }//GEN-LAST:event_formMouseEntered
 
-        try {
-            // Default to system-specific L&F
-            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
-        } catch (Exception e) {
-            // handle exception
-        }
-
-        java.awt.EventQueue.invokeLater(new Runnable() {
-
-            public void run() {
-                new OpticalRayTracer().setVisible(true);
-            }
-        });
-    }
+    private void formMouseExited(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseExited
+        mouse_present_test(false);
+    }//GEN-LAST:event_formMouseExited
 
+    private void sv_zeroRotCheckBoxMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_sv_zeroRotCheckBoxMouseClicked
+        readControls();
+    }//GEN-LAST:event_sv_zeroRotCheckBoxMouseClicked
     // Variables declaration - do not modify//GEN-BEGIN:variables
     private javax.swing.JButton barrierColorButton;
     private javax.swing.JButton baselineColorButton;
@@ -1879,6 +1967,7 @@ final public class OpticalRayTracer extends javax.swing.JFrame implements Clipbo
     protected javax.swing.JTextField sv_xTargetPlaneTextField;
     protected javax.swing.JTextField sv_yEndTextField;
     protected javax.swing.JTextField sv_yStartTextField;
+    protected javax.swing.JCheckBox sv_zeroRotCheckBox;
     protected javax.swing.JCheckBox symmCheckBox;
     private javax.swing.JLabel thicknessLabel1;
     private javax.swing.JButton undoButton;
