@@ -96,7 +96,7 @@ final public class RayTraceComputer {
 
             parent.sv_lensList.add(new Lens(parent, this, 0, 0, 2, 6, 6, 0, ior, cf, cf, disp, leftHyp, rightHyp, symmetrical));
             parent.sv_lensList.add(new Lens(parent, this, 4, 0, 2, -6, -6, 0, ior, cf, cf, disp, leftHyp, rightHyp, symmetrical));
-        //rayTraceProcess(true);
+            //rayTraceProcess(true);
         }
     }
 
@@ -114,6 +114,7 @@ final public class RayTraceComputer {
         drawScaledLine(xt, max * 100, ix, iy, g, parent.beamWidth, true);
         if (parent.beamCount > 1) {
             double fact = (max - min) / ((double) parent.beamCount - 1);
+            //double fact = (max - min) / ((double) parent.beamCount);
             double y;
             double xSource = parent.xBeamSourceRefPlane;
             double xTarget = parent.xBeamTargetRefPlane;
@@ -121,17 +122,23 @@ final public class RayTraceComputer {
             // this value must not be zero
             ba = (ba == 0.0) ? 1e-12 : ba;
             for (int i = 0; (y = min + i * fact) <= max; i++) {
-                double my = (parent.divergingSource) ? 0 : y;
-                my -= (xTarget - xSource) * Math.tan(ba);
+                double mya = (parent.divergingSource) ? 0 : y;
+                double myb = (parent.divergingSource) ? ((parent.rotXZero) ? y * (xTarget - xSource) / xSource : y) : y;
+                mya += Math.tan(ba) * ((parent.rotXZero)
+                        ? xSource
+                        : (xSource - xTarget));
+                myb += Math.tan(ba) * ((parent.rotXZero)
+                        ? xTarget
+                        : 0);
                 if (parent.dispersionBeams > 1) {
                     for (int j = 0; j < parent.dispersionBeams; j++) {
                         double wvl = (((double) j / (parent.dispersionBeams - 1)) * (650.0 - 400.0)) + 400.0;
                         ColorPos cp = createColorForWvl(wvl);
                         MyColor beamColor = new MyColor(cp.r, cp.g, cp.b);
-                        traceOneRay(width, height, xSource, my, xTarget, y, g, parent.sv_lensList, beamColor, wvl);
+                        traceOneRay(width, height, xSource, mya, xTarget, myb, g, parent.sv_lensList, beamColor, wvl);
                     }
                 } else { // no dispersion
-                    traceOneRay(width, height, xSource, my, xTarget, y, g, parent.sv_lensList, pbcol, 0);
+                    traceOneRay(width, height, xSource, mya, xTarget, myb, g, parent.sv_lensList, pbcol, 0);
                 }
             }
         }
@@ -245,7 +252,8 @@ final public class RayTraceComputer {
         } // while rays++ < parent.maxIntersections
     }
 
-    void drawBaseline(Graphics g) {
+    void drawBaselines(Graphics g) {
+        // y = 0 line
         double x1 = 0;
         double x2 = parent.xSize;
         double y = 0;
@@ -254,6 +262,14 @@ final public class RayTraceComputer {
         g.setColor(parent.sv_yBaselineColor.getColor());
         drawScaledLine(dx1.v, y, ix, iy, g, 1, false);
         drawScaledLine(dx2.v, y, ix, iy, g, 1, true);
+        // x = 0 line
+        double y1 = 0;
+        double y2 = parent.ySize;
+        double x = 0;
+        displayToSpaceOffset(x, y1, dx, dy1);
+        displayToSpaceOffset(x, y2, dx, dy2);
+        drawScaledLine(x, dy1.v, ix, iy, g, 1, false);
+        drawScaledLine(x, dy2.v, ix, iy, g, 1, true);
     }
 
     void drawGrid(Graphics g) {
@@ -454,8 +470,8 @@ final public class RayTraceComputer {
     void scalePoint(double x, double y, MutableInt dx, MutableInt dy) {
         //CartesianInt ci = new CartesianInt();
         spaceToDisplay(x, y, dx, dy);
-    //ci.x = (int) cd.x;
-    //ci.y = (int) cd.y;
+        //ci.x = (int) cd.x;
+        //ci.y = (int) cd.y;
     }
 
     void drawScaledPoint(double x, double y, double radius, Graphics g, Color col) {
